from ctypes import c_int, c_void_p, py_object, c_bool, CFUNCTYPE, Structure, POINTER

from .lib import lib

from .packets import Packet

class HandlerData (Structure):
	_fields_ = [
		("handler_id", c_int),

		("data", py_object),
		("packet", POINTER (Packet))
	]

HandlerMethod = CFUNCTYPE (None, c_void_p)

HandlerDataCreate = CFUNCTYPE (py_object, c_void_p)
HandlerDataDelete = CFUNCTYPE (None, c_void_p)

handler_delete = lib.handler_delete
handler_delete.argtypes = [c_void_p]

handler_create = lib.handler_create
handler_create.argtypes = [HandlerMethod]
handler_create.restype = c_void_p

handler_create_with_id = lib.handler_create_with_id
handler_create_with_id.argtypes = [c_int, HandlerMethod]
handler_create_with_id.restype = c_void_p

handler_set_data = lib.handler_set_data
handler_set_data.argtypes = [c_void_p, c_void_p]
handler_set_data.restype = c_void_p

handler_set_data_create = lib.handler_set_data_create
handler_set_data_create.argtypes = [c_void_p, HandlerDataCreate, c_void_p]

handler_set_data_delete = lib.handler_set_data_delete
handler_set_data_delete.argtypes = [c_void_p, HandlerDataDelete]

handler_set_direct_handle = lib.handler_set_direct_handle
handler_set_direct_handle.argtypes = [c_void_p, c_bool]



from ctypes import c_int, c_uint8, c_uint16, c_uint32, c_size_t, c_void_p, c_char_p, c_bool, Structure

from .lib import lib

PacketType = c_int

PACKET_TYPE_NONE = 0
PACKET_TYPE_CERVER = 1 
PACKET_TYPE_CLIENT = 2
PACKET_TYPE_ERROR = 3
PACKET_TYPE_REQUEST = 4
PACKET_TYPE_AUTH = 5
PACKET_TYPE_GAME = 6
PACKET_TYPE_APP = 7
PACKET_TYPE_APP_ERROR = 8
PACKET_TYPE_CUSTOM = 9
PACKET_TYPE_TEST = 10
PACKET_TYPE_BAD = 11

ProtocolID = c_uint32

class ProtocolVersion (Structure):
	_fields_ = [
		("major", c_uint16),
		("minor", c_uint16)
	]

class PacketVersion (Structure):
	_fields_ = [
		("protocol_id", ProtocolID),
		("protocol_version", ProtocolVersion)
	]

class PacketHeader (Structure):
	_fields_ = [
		("packet_type", PacketType),
		("packet_size", c_size_t),

		("handler_id", c_uint8),

		("request_type", c_uint32),

		("sock_fd", c_uint16)
	]

class Packet (Structure):
	_fields_ = [
		("cerver", c_void_p),
		("client", c_void_p),
		("connection", c_void_p),
		("lobby", c_void_p),

		("packet_type", PacketType),
		("req_type", c_uint32),

		("data_size", c_size_t),
		("data", c_void_p),
		("data_ptr", c_char_p),
		("data_end", c_char_p),
		("data_ref", c_bool),

		("remaining_data", c_size_t),

		("header", PacketHeader),

		("version", PacketVersion),
		
		("packet_size", c_size_t),
		("packet", c_void_p),
		("packet_ref", c_bool)
	]

packet_send_request = lib.packet_send_request
packet_send_request.argtypes = [PacketType, c_uint32, c_void_p, c_void_p, c_void_p, c_void_p]
packet_send_request.restype = c_uint8



import distutils.util

from ..files import IMAGE_TYPE_PNG, IMAGE_TYPE_JPEG, files_image_get_type

from .request import http_request_multi_parts_get_value
from .request import http_request_multi_parts_get_saved_filename

def validate_body_value_exists (body, value, errors):
	result = None

	if (value in body):
		if (len (body[value]) > 0):
			result = body[value]

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_body_value (body, value, min_len, max_len, errors):
	result = None

	found = validate_body_value_exists (body, value, errors)
	if (found):
		value_len = len (found)
		if (value_len >= min_len and value_len <= max_len):
			result = found

		else:
			errors[value] = (
				"Field {0} must be between {1} and {2} characters long."
				.format	(value, min_len, max_len)
			)

	return result

def validate_mparts_exists (request, value, errors):
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		if (len (found) > 0):
			result = found

	if (result is None):
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_value (request, value, min_len, max_len, errors):
	result = None

	found = validate_mparts_exists (request, value, errors)
	if (found):
		value_len = len (found)
		if (value_len >= min_len and value_len <= max_len):
			result = found

		else:
			errors[value] = (
				"Field {0} must be between {1} and {2} characters long."
				.format	(value, min_len, max_len)
			)

	return result

def validate_mparts_int (request, value, errors):
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = int (found)
		except ValueError:
			errors[value] = f"Field {value} is invalid."

	else:
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_int_with_default (
	request, value, default, errors
):
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = int (found)
		except ValueError:
			pass

	return result

def validate_mparts_float (request, value, errors):
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = float (found)
		except ValueError:
			errors[value] = f"Field {value} is invalid."

	else:
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_float_with_default (
	request, value, default, errors
):
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = float (found)
		except ValueError:
			pass

	return result

def validate_mparts_bool (request, value, errors):
	result = None

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = bool (distutils.util.strtobool (found))
		except ValueError:
			errors[value] = f"Field {value} is invalid."

	else:
		errors[value] = f"Field {value} is required."

	return result

def validate_mparts_bool_with_default (
	request, value, default, errors
):
	result = default

	found = http_request_multi_parts_get_value (request, value.encode ("utf-8"))
	if (found):
		found = found.decode ("utf-8")
		try:
			result = bool (distutils.util.strtobool (found))
		except ValueError:
			pass

	return result

def validate_mparts_file_is_image (request, image, errors):
	result = False
	img_type = None

	# get the image filename
	filename = http_request_multi_parts_get_saved_filename (
		request, image.encode ("utf-8")
	)

	if (filename):
		# validate file and get extension
		img_type = files_image_get_type (filename)
		if (img_type == IMAGE_TYPE_PNG or img_type == IMAGE_TYPE_JPEG):
			result = True

		else:
			errors[image] = f"File {image} is not png or jpeg."

	else:
		errors[image] = f"File {image} is missing."

	return result, img_type, filename



from .alg import *
from .content import *
from .headers import *
from .http import *
from .multipart import *
from .query import *
from .request import *
from .response import *
from .route import *
from .status import *
from .validation import *
